"use strict";

exports.__esModule = true;
exports.default = validateChildKeys;

var _host = _interopRequireDefault(require("./vdom/host"));

var _warning = _interopRequireDefault(require("./warning"));

var _types = require("./types");

var _getRenderErrorInfo = _interopRequireDefault(require("./getRenderErrorInfo"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Warn if there's no key explicitly set on dynamic arrays of children or
 * object keys are not valid. This allows us to keep track of children between
 * updates.
 */
var ownerHasKeyUseWarning = {};

function getCurrentComponentErrorInfo(parentType) {
  var info = (0, _getRenderErrorInfo.default)();

  if (!info) {
    var parentName = typeof parentType === 'string' ? parentType : parentType.displayName || parentType.name;

    if (parentName) {
      info = "\n\nCheck the top-level render call using <" + parentName + ">.";
    }
  }

  return info;
}

function isValidElement(object) {
  return typeof object === 'object' && object !== null && object.type && !!object.props;
}

function validateExplicitKey(element, parentType) {
  if (element.__validated || element.key != null) {
    return;
  }

  element.__validated = true;
  var currentComponentErrorInfo = getCurrentComponentErrorInfo(parentType);

  if (ownerHasKeyUseWarning[currentComponentErrorInfo]) {
    return;
  }

  ownerHasKeyUseWarning[currentComponentErrorInfo] = true; // Usually the current owner is the offender, but if it accepts children as a
  // property, it may be the creator of the child that's responsible for
  // assigning it a key.

  var childOwner = '';

  if (element && element._owner && element._owner !== _host.default.owner) {
    // Give the component that originally created this child.
    childOwner = " It was passed a child from " + element._owner.__getName() + ".";
  }

  (0, _warning.default)('Each child in a list should have a unique "key" prop.%s%s', currentComponentErrorInfo, childOwner);
}

function validateChildKeys(node, parentType) {
  if (!(0, _types.isObject)(node)) {
    return;
  }

  if ((0, _types.isArray)(node)) {
    for (var i = 0; i < node.length; i++) {
      var child = node[i];

      if (isValidElement(child)) {
        validateExplicitKey(child, parentType);
      }
    }
  } else if (isValidElement(node)) {
    node.__validated = true;
  } // rax isn't support iterator object as element children
  // TODO: add validate when rax support iterator object as element.

}